/*
    Magnifier II - An image viewer with a magnifier
    Copyright © 2014-2017 Harry Whitfield

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the Free
    Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    MA  02110-1301  USA

    Magnifier II - web widget version 1.0 with world file functions
    22 November, 2017
    Copyright © 2014-2017 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint browser */

/*global newImage, newSelector, addToMenu, moveObj */

/*property
    forEach, onchange, onmousedown, onmouseup, opacity, selectedIndex, style,
    value
*/

//////////////////////////////////////// GUI Code ////////////////////////////////////////

var imageBase = "Resources/Images/";
var prefpanexo = 511;
var prefpaneyo = 137;
var prefPane = newImage(prefpanexo, prefpaneyo, 512, 480, imageBase + "prefPane.png", 21, 0.0);
var cancel = newImage(prefpanexo + 277, prefpaneyo + 355, 76, 30, imageBase + "Cancel.png", 22, 0.0);
var save = newImage(prefpanexo + 357, prefpaneyo + 357, 76, 30, imageBase + "Save.png", 22, 0.0);
var deltaY = 1000;

/////////////////////////////////////// Menus Code ///////////////////////////////////////

var utmZones = [
	"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12",
	"13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24",
	"25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36",
	"37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48",
	"49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60"
];

var hemispheres = [
	"Northern",
	"Southern"
];

var coordinatess = [
	"UTM eastings and northings",
	"Lat/Long Degrees",
	"Lat/Long Degrees, Mins and Secs"
];

var datums = [
	"WGS84",
	"GRS80",
	"NAD27"
];

/*
var utmZonePref = "31";								// default menu setting
var hemispherePref = "Northern";					// default menu setting
var coordinatesPref = "UTM eastings and northings";	// default menu setting
var datumPref = "WGS84"; 							// default menu setting
*/

var pMenuArray = [									// menu items for four menus
	utmZones,
	hemispheres,
	coordinatess,
	datums
];

var pMenuTitleArray = [								// menu tooltips
	"UTM Zone",
	"Hemisphere",
	"Coordinates for displayed waypoints",
	"Geodetic datum for conversions to latitude and longitude"
];

var pMenuSelectedIndex = [30, 0, 0, 0];				// default menu settings

var pMenu = [];										// declare the Menu array

pMenu[0] = newSelector(prefpanexo + 166, prefpaneyo + 63, 110, 20, "", 22, 0.0);		// create the menu selectors
pMenu[1] = newSelector(prefpanexo + 166, prefpaneyo + 175, 110, 20, "", 22, 0.0);
pMenu[2] = newSelector(prefpanexo + 166, prefpaneyo + 227, 110, 20, "", 22, 0.0);
pMenu[3] = newSelector(prefpanexo + 166, prefpaneyo + 291, 110, 20, "", 22, 0.0);

function makePrefPane(visible) {
	"use strict";
	if (visible) {
		moveObj(pMenu[0], prefpanexo + 166, prefpaneyo + 63);
		moveObj(pMenu[1], prefpanexo + 166, prefpaneyo + 175);
		moveObj(pMenu[2], prefpanexo + 166, prefpaneyo + 227);
		moveObj(pMenu[3], prefpanexo + 166, prefpaneyo + 291);
		moveObj(cancel, prefpanexo + 277, prefpaneyo + 355);
		moveObj(save, prefpanexo + 357, prefpaneyo + 357);
		moveObj(prefPane, prefpanexo, prefpaneyo);
	} else {
		moveObj(pMenu[0], prefpanexo + 166, prefpaneyo + 63 - deltaY);
		moveObj(pMenu[1], prefpanexo + 166, prefpaneyo + 175 - deltaY);
		moveObj(pMenu[2], prefpanexo + 166, prefpaneyo + 227 - deltaY);
		moveObj(pMenu[3], prefpanexo + 166, prefpaneyo + 291 - deltaY);
		moveObj(cancel, prefpanexo + 277, prefpaneyo + 355 - deltaY);
		moveObj(save, prefpanexo + 357, prefpaneyo + 357 - deltaY);
		moveObj(prefPane, prefpanexo, prefpaneyo - deltaY);
	}
}

var gUtmZone;		// preference values set elsewhere
var gHemisphere;
var gCoordinates;
var gDatum;

var pUtmZone;		// preference values set here
var pHemisphere;
var pCoordinates;
var pDatum;

pMenu[0].onchange = function () {
	"use strict";
	pUtmZone = pMenu[0].value;
};

pMenu[1].onchange = function () {
	"use strict";
	pHemisphere = pMenu[1].value;
};

pMenu[2].onchange = function () {
	"use strict";
	pCoordinates = String(pMenu[2].selectedIndex);
};

pMenu[3].onchange = function () {
	"use strict";
	pDatum = pMenu[3].value;
};

function openPrefPane() {
	"use strict";
    pMenu[0].value = gUtmZone;
    pMenu[1].value = gHemisphere;
    pMenu[2].value = coordinatess[Number(gCoordinates)];
    pMenu[3].value = gDatum;

    pMenu[0].selectedIndex = Number(gUtmZone) - 1;
    pMenu[1].selectedIndex = (
    	gHemisphere === hemispheres[0]
    	? 0
    	: 1
    );
    pMenu[2].selectedIndex = Number(gCoordinates);
    pMenu[3].selectedIndex = (
    	gDatum === datums[0]
    	? 0
    	: (
    		gDatum === datums[1]
    		? 1
    		: 2
    	)
    );

 	pUtmZone = gUtmZone;
	pHemisphere = gHemisphere;
	pCoordinates = gCoordinates;
	pDatum = gDatum;

	pMenu.forEach(function (ele) {
		ele.style.opacity = 1.0;
	});
	cancel.style.opacity = 1.0;
	save.style.opacity = 1.0;
	prefPane.style.opacity = 1.0;
	makePrefPane(true);
}

function closePrefPane(saving) {
	"use strict";

	if (saving) {
		gUtmZone = pUtmZone;
		gHemisphere = pHemisphere;
		gCoordinates = pCoordinates;
		gDatum = pDatum;
	}

	pMenu.forEach(function (ele) {
		ele.style.opacity = 0.0;
	});
	cancel.style.opacity = 0.0;
	save.style.opacity = 0.0;
	prefPane.style.opacity = 0.0;
	makePrefPane(false);
}

cancel.onmousedown = function () {
	"use strict";
    cancel.style.opacity = "0.5";
};
cancel.onmouseup = function () {
	"use strict";
    cancel.style.opacity = "1.0";
    closePrefPane(false);
};

save.onmousedown = function () {
	"use strict";
    save.style.opacity = "0.5";
};
save.onmouseup = function () {
	"use strict";
    save.style.opacity = "1.0";
    closePrefPane(true);
};

(function () {										// initialise the menus
	"use strict";
	pMenuArray.forEach(function (ele, i) {
		addToMenu(pMenu[i], ele);
		pMenu[i].selectedIndex = pMenuSelectedIndex[i];
//		pMenu[i].title = pMenuTitleArray[i];		// tooltips not needed
	});
}());

makePrefPane(false);

/////////////////////////////////////// Other Code ///////////////////////////////////////
