/*
	Magnifier II - An image viewer with a magnifier
	Copyright © 2014-2017 Harry Whitfield

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA	02110-1301	USA

	Magnifier II - web widget version 1.0 with world file functions
	22 November, 2017
	Copyright © 2014-2017 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*jslint browser, devel */

/*property
	A, B, C, D, E, F, coordinates, datum, eastings, floor, hemisphere,
	lastIndexOf, lat, latitude, length, lon, longitude, name, northings,
	opacity, push, split, stringify, style, substring, toFixed, type, utmZone,
	value, writeFile, x, y, zone
*/

var utmToLL;
var xtn;
var wpFolder;
var filesystem;
var openPrefPane;

var utmZonePref;		// set in webstorage.js
var hemispherePref;
var coordinatesPref;
var datumPref;

var gImagePath = null;
var gOutputFolder = wpFolder;

var eprint = alert;
var sprint = alert;
var beep = function () {
	"use strict";
	return;
};
//var writeFile = alert;

//////////////////////////////////// world files /////////////////////////////////////////

var gWorldPath = null;

var utmT = null;

var prec1 = 4; // precision for seconds of arc
var prec2 = 3; // precision for eastings, northings
var prec3 = 8; // precision for convergence
var prec4 = 7; // precision for numeric latitude/longitude

var waypoints = [];

function folder(path) {
	"use strict";
	var idx = path.lastIndexOf("/");

	if (idx >= 0) {
		return path.substring(0, idx);
	}
	return path;
}

function filename(path) {
	"use strict";
	var idx = path.lastIndexOf("/");

	if (idx >= 0) {
		return path.substring(idx + 1);
	}
	return path;
}

function stem(filename) {
	"use strict";
	var idx = filename.lastIndexOf(".");

	if (idx >= 0) {
		return filename.substring(0, idx);
	}
	return filename;
}

function readWorldFile(data) {
	"use strict";
	var t = {};

	if (data !== null) {
		data = data.split(/\r\n?|\n/);
		t.A = parseFloat(data[0]);	// x component of the pixel width (x-scale)		mx * cos(t)
		t.D = parseFloat(data[1]);	// y component of the pixel width (y-skew)		mx * sin(t)
		t.B = parseFloat(data[2]);	// x component of the pixel height (x-skew)		my * (k * cos(t) - sin(t))	where k = tan(skew)
		t.E = parseFloat(data[3]);	// y component of the pixel height (y-scale)	-1 * my * (k * sin(t) + cos(t))
		t.C = parseFloat(data[4]);	// translation in x direction					the skew angle is measured from the y-axis
		t.F = parseFloat(data[5]);	// translation in y direction					and t = rotation angle measured from x-axis
		return t;
	}
	return null;
}

function utmTrans(x, y, t) {
	"use strict";
	var x1 = t.A * x + t.B * y + t.C;
	var y1 = t.D * x + t.E * y + t.F;

	return [x1, y1];
}

/*
var utmZonePref = "31";			// All strings
var hemispherePref = "Northern";
var coordinatesPref = "0";
var datumPref = "WGS84";
*/

function setOpacity(obj, opacity) {
	"use strict";
	obj.style.opacity = opacity;
}

/*
function checkWorldFile(setwfmode) {
	"use strict";
	var worldName,
		imageName;

	if (gImagePath && gWorldPath) {
		// check that the world file matches the image file
		worldName = stem(filename(gWorldPath));
		imageName = stem(filename(gImagePath));
		if (imageName !== worldName) {
			alert("The world file does not match the current image.");
			utmT = null;
			setwfmode(false);
			gWorldPath = null;
		}
	}
}
*/

function getWorldFile(data, path, setwfmode) {
	"use strict";
	var worldName;
	var imageName;
//		mapPrefs = null;

	if (gImagePath !== null) {
		//gWorldPath = path;
		//sprint("Current Image: " + gImagePath);
		// check that the world file matches the image file
		worldName = stem(filename(gWorldPath));
		imageName = stem(filename(gImagePath));
		if (imageName === worldName) {
			utmT = readWorldFile(data);
			if (utmT !== null) {
//				sprint("World File:	   " + path);
//				theInput.value = "Enter Waypoint Here";
				setwfmode(true);
				if (coordinatesPref !== "0") {
					openPrefPane();
					/*
					mapPrefs = getMapPrefs(
						utmZonePref,
						hemispherePref,
						coordinatesPref,
						datumPref
					);
					if (mapPrefs !== null) {
						utmZonePref = mapPrefs.utmZone;
						hemispherePref = mapPrefs.hemisphere;
						coordinatesPref = mapPrefs.coordinates;
						datumPref = mapPrefs.datum;
					}
					*/
				}

			} else {
				beep();
				alert("Failed to read World File: " + path);
//				sprint("Failed to read World File: " + path);
				setwfmode(false);
			}
		} else {
			beep();
			alert("The world file does not match the current image.");
//			sprint("The world file does not match the current image.");
			utmT = null;
			setwfmode(false);
		}
	} else {
		utmT = null;
		setwfmode(false);
	}
}

/*
function frameUtm() {
	"use strict";
	var path = chooseFile([".jpgw", ".jgw", ".tifw", ".tfw", ".pngw", ".pgw", ".gifw", ".gfw", ".bmpw", ".bpw"]);

	getWorldFile(path);
}

function findWorldFile() {
	"use strict";
	var imageName = stem(filename(gImagePath)),
		imageExtn = xtn(gImagePath),
		imageDir = folder(gImagePath),
		test = function () {
			var path = imageDir + "/" + imageName + imageExtn + "w";

			if (filesystem.itemExists(path)) {
				getWorldFile(path);
			} else {
				path = imageDir + "/" + imageName + "." + imageExtn[1] + imageExtn[3] + "w";
				if (filesystem.itemExists(path)) {
					getWorldFile(path);
				}
			}
		};

//	theInput.value = "";

	switch (imageExtn) {
	case ".jpg":
		test();
		break;
	case ".tif":
		test();
		break;
	case ".png":
		test();
		break;
	case ".gif":
		test();
		break;
	case ".bmp":
		test();
		break;
	}
}
*/

function get_lat(K) { // output angle K (degrees)
	"use strict";
	var d;
	var dd;
	var m;
	var mm;
	var s;
	var ss;
	var ns;
	var coordinatesX = coordinatesPref;

	ns = "N";
	if (K < 0) {
		K = -K;
		ns = "S";
	}
//	K = 180.0 * K / Math.PI;
	if (coordinatesX === "1") {
		d = K.toFixed(prec4);
		dd = String(Math.floor(Number(d)));
		if (dd.length === 1) {
			d = "0" + d;
		}
		return d + " " + ns;
	}
	d = Math.floor(K);
	K = 60.0 * (K - d);
	m = Math.floor(K);
	s = 60.0 * (K - m);

	dd = String(d);
	if (dd.length === 1) {
		d = "0" + d;
	}

	mm = String(m);
	if (mm.length === 1) {
		m = "0" + m;
	}

	s = s.toFixed(prec1);
	ss = String(Math.floor(Number(s)));
	if (ss.length === 1) {
		s = "0" + s;
	}

	return d + " " + m + " " + s + " " + ns;
}

function get_long(K) { // output angle K (degrees)
	"use strict";
	var d;
	var ddd;
	var m;
	var mm;
	var s;
	var ss;
	var ew;
	var coordinatesX = coordinatesPref;

	ew = "E";
	if (K < 0) {
		K = -K;
		ew = "W";
	}
//	K = 180.0 * K / Math.PI;
	if (coordinatesX === "1") {
		d = K.toFixed(prec4);
		ddd = String(Math.floor(Number(d)));
		while (ddd.length < 3) {
			ddd = "0" + ddd;
			d = "0" + d;
		}
		return d + " " + ew;
	}
	d = Math.floor(K);
	K = 60.0 * (K - d);
	m = Math.floor(K);
	s = 60.0 * (K - m);

	ddd = String(d);
	while (ddd.length < 3) {
		ddd = "0" + ddd;
		d = "0" + d;
	}

	mm = String(m);
	if (mm.length === 1) {
		m = "0" + m;
	}

	s = s.toFixed(prec1);
	ss = String(Math.floor(Number(s)));
	if (ss.length === 1) {
		s = "0" + s;
	}

	return d + " " + m + " " + s + " " + ew;
}

function utmZone(lat, lon) {
	"use strict";
	var bands = "CDEFGHJKLMNPQRSTUVWX";
	var zone = Math.floor((lon + 180) / 6) + 1;
	var band;

	if ((-80 <= lat) && (lat < 80)) {
		band = bands[Math.floor((lat + 80) / 8)];
	} else if ((80 <= lat) && (lat < 84)) {
		band = "X";
	} else {
		return;
	}

	if ((band === "V") && (3 <= lon) && (lon < 12)) {	// Norway)
		zone = 32;
	}

	if (band === "X") {
		if ((0 <= lon) && (lon < 9)) {
			zone = 31;
		} else if ((9 <= lon) && (lon < 21)) {	// Svalbard
			zone = 33;
		} else if ((21 <= lon) && (lon < 33)) {
			zone = 35;
		} else if ((33 <= lon) && (lon < 42)) {
			zone = 37;
		}
	}

	zone = String(zone);
	if (zone.length === 1) {
		zone = "0" + zone;
	}

	return zone + band;
}

function coordMap(x, y, shiftKey, altKey, theInputData, theOutputArea) {
	"use strict";
	var s;
	var utm;
	var zone;
	var latLon;
	var eastings;
	var northings;
	var latitude;
	var longitude;
	var waypoint = {};
	var datumX = datumPref;
	var coordinatesX = coordinatesPref;
	var utmZoneX = utmZonePref;
	var hemisphereX = hemispherePref;

	if ((utmT !== null) && (shiftKey || altKey)) {
		waypoint.x = x;
		waypoint.y = y;
		waypoint.name = theInputData;
		waypoint.type = Number(coordinatesX);
		utm = utmTrans(x, y, utmT);
		if (coordinatesX === "0") {
			eastings = utm[0].toFixed(3);
			waypoint.eastings = eastings;
			northings = utm[1].toFixed(3);
			waypoint.northings = northings;
			waypoint.zone = utmZoneX + " " + hemisphereX.substring(0, 5);
			s = "[" + eastings + ", " + northings + "]";	//	+ " " + waypoint.zone;
		} else {
			latLon = utmToLL(utm[1], utm[0], Number(utmZoneX), hemisphereX.substring(0, 1), datumX);
			zone = utmZone(latLon.lat, latLon.lon);
			latitude = get_lat(latLon.lat);
			waypoint.latitude = latitude;
			longitude = get_long(latLon.lon);
			waypoint.longitude = longitude;
			waypoint.zone = zone;
			s = "[" + latitude + ", " + longitude + "]";
			if (zone) {
				s += " (" + zone + ")";
			}
		}
		waypoint.datum = datumX;
		if (altKey) {
			//sprint("[" + x + ", " + y + "] => " + s + " " + theInputData);
			waypoints.push(waypoint);
			theOutputArea.value = JSON.stringify(waypoints);
//			sprint(JSON.stringify(waypoints));
		}
	} else {
		s = "[" + x + ", " + y + "]";
	}
	return s;
}

function clearWaypoints() { // needs a button
	"use strict";
//	alert("clearWaypoints");
	waypoints = [];
}

function saveWaypoints() {	// needs a button
	"use strict";
	var fldr;
	var imageName;

	if (waypoints.length > 0) {
		fldr = gOutputFolder;
		if (fldr && filesystem) {	// implies that filesystem functions are available
			imageName = stem(gImagePath);
			filesystem.writeFile(fldr + "/" + imageName + ".json", JSON.stringify(waypoints));
		} else {
			alert("filesystem functions are not available");
		}
	} else {
		alert("There are no waypoint records.");
	}
}

//////////////////////////////////////////////////////////////////////////////////////////
